@@ -1,6 +1,6 @@
 /* common.c - common functions used by daisy-player and eBook-speaker.
  *
- * Copyright (C)2020 J. Lemmens
+ * Copyright (C)2021 J. Lemmens
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -19,8 +19,6 @@
 
 #include "daisy.h"
 
-char sink_info[10][100];
-
 void open_xml_file (misc_t *misc, my_attribute_t *my_attribute,
                     daisy_t *daisy, char *xml_file, char *anchor)
 {
@@ -56,7 +54,9 @@ void open_xml_file (misc_t *misc, my_attribute_t *my_attribute,
       failure (misc, str, e);
    } // if
    if (! *anchor)
+   {
       return;
+   } // if
 
 // skip to anchor           
    while (1)
@@ -72,7 +72,8 @@ char *convert_URL_name (misc_t *misc, char *file)
 {
    int i, j;
 
-   *misc->str = j = 0;
+   *misc->str = 0;
+   j = 0;
    for (i = 0; i < (int) strlen (file) - 2; i++)
    {
       if (file[i] == '%' && isxdigit (file[i + 1]) && isxdigit (file[i + 2]))
@@ -104,7 +105,7 @@ void failure (misc_t *misc, char *str, int e)
    printf ("%s: %s\n", str, strerror (e));
    fflush (stdout);
    remove_tmp_dir (misc);
-   switch (system ("reset"));
+   system ("reset");
    _exit (EXIT_FAILURE);
 } // failure
 
@@ -165,7 +166,7 @@ void get_real_pathname (char *dir, char *search_str, char *found)
    free (namelist);
 } // get_real_pathname
 
-char *get_dir_content (misc_t *misc, char *dir_name, char *search_str)
+char *find_ncx_opf (misc_t *misc, char *dir_name, char *search_str)
 {
    char *found;
    DIR *dir;
@@ -179,10 +180,14 @@ char *get_dir_content (misc_t *misc, char *dir_name, char *search_str)
       failure (misc, "readdir ()", errno);
    do
    {
+      int offset;
+
       if (strcmp (entry->d_name, ".") == 0 ||
           strcmp (entry->d_name, "..") == 0)
          continue;
-      if (strcasestr (entry->d_name, search_str) && *search_str)
+      offset = (int) strlen (entry->d_name) - 4;
+      if (strcasecmp (entry->d_name + offset, search_str) == 0 &&
+          *search_str)
       {
          found = malloc (strlen (dir_name) + strlen (entry->d_name) + 5);
          sprintf (found, "%s/%s\n", dir_name, entry->d_name);
@@ -200,7 +205,7 @@ char *get_dir_content (misc_t *misc, char *dir_name, char *search_str)
          path = malloc (strlen (dir_name) + strlen (entry->d_name) + 10);
          sprintf (path, "%s/%s", dir_name, entry->d_name);
          found = malloc (MAX_STR);
-         strcpy (found, get_dir_content (misc, path, search_str));
+         strcpy (found, find_ncx_opf (misc, path, search_str));
          if (strcasestr (found, search_str) && *search_str)
          {
             free (path);
@@ -210,7 +215,7 @@ char *get_dir_content (misc_t *misc, char *dir_name, char *search_str)
       } // if
    } while ((entry = readdir (dir)));
    return "";
-} // get_dir_content
+} // find_ncx_opf
 
 void find_index_names (misc_t *misc)
 {
@@ -219,10 +224,10 @@ void find_index_names (misc_t *misc)
    get_real_pathname (misc->daisy_mp, "ncc.html", misc->ncc_html);
    *misc->ncx_name = 0;
    strcpy (misc->ncx_name,
-            get_dir_content (misc, misc->daisy_mp, ".ncx"));
+            find_ncx_opf (misc, misc->daisy_mp, ".ncx"));
    *misc->opf_name = 0;
    strcpy (misc->opf_name,
-            get_dir_content (misc, misc->daisy_mp, ".opf"));
+            find_ncx_opf (misc, misc->daisy_mp, ".opf"));
    dirc = strdup (misc->opf_name);
    free (misc->daisy_mp);
    misc->daisy_mp = strdup (dirname (dirc));
@@ -230,7 +235,7 @@ void find_index_names (misc_t *misc)
 
 void kill_player (misc_t *misc)
 {
-   while (kill (misc->player_pid, SIGKILL) == 0);
+   while (kill (misc->player_pid, SIGTERM) == 0);
    misc->player_pid = -2;
    unlink (misc->eBook_speaker_txt);
    unlink (misc->tmp_wav);
@@ -262,7 +267,7 @@ int handle_ncc_html (misc_t *misc, my_attribute_t *my_attribute,
    dirc = strdup (misc->daisy_mp);
    misc->daisy_mp = strdup (dirname (dirc));
    free (dirc);
-   strncpy (misc->daisy_version, "2.02", 4);
+   strcpy (misc->daisy_version, "2.02");
    doc = htmlParseFile (misc->ncc_html, "UTF-8");
    ncc = xmlReaderWalker (doc);
    misc->total_items = 0;
@@ -276,7 +281,7 @@ int handle_ncc_html (misc_t *misc, my_attribute_t *my_attribute,
           strcasecmp (misc->tag, "h4") == 0 ||
           strcasecmp (misc->tag, "h5") == 0 ||
           strcasecmp (misc->tag, "h6") == 0)
-      {
+      {                          
          misc->total_items++;
       } // if
    } // while
@@ -317,7 +322,7 @@ int get_meta_attributes (xmlTextReaderPtr parse, xmlTextWriterPtr writer)
                      (const unsigned char *) "content");
       if (strcasecmp (name, "title") == 0)
       {
-         xmlTextWriterWriteString (writer, (const xmlChar *)content);
+         xmlTextWriterWriteString (writer, (const xmlChar *) content);
          return 1;
       } // if
       if (xmlTextReaderRead (parse) == 0)
@@ -340,18 +345,14 @@ void create_ncc_html (misc_t *misc)
    xmlTextWriterSetIndentString (writer, BAD_CAST "   ");
    xmlTextWriterStartDocument (writer, NULL, NULL, NULL);
    xmlTextWriterStartElement (writer, BAD_CAST "html");
-   xmlTextWriterWriteString (writer, BAD_CAST "\n");
    xmlTextWriterStartElement (writer, BAD_CAST "head");
-   xmlTextWriterWriteString (writer, BAD_CAST "\n   ");
    xmlTextWriterEndElement (writer);
    xmlTextWriterStartElement (writer, BAD_CAST "body");
-   xmlTextWriterWriteString (writer, (const xmlChar *)"\n");
    total = scandir (get_current_dir_name (), &namelist, namefilter,
                     alphasort);
    for (n = 0; n < total; n++)
    {
       xmlTextWriterStartElement (writer, BAD_CAST "h1");
-      xmlTextWriterWriteString (writer, BAD_CAST "\n");
       xmlTextWriterStartElement (writer, BAD_CAST "a");
       xmlTextWriterWriteFormatAttribute (writer, BAD_CAST "href",
                                          "%s", namelist[n]->d_name);
@@ -383,7 +384,6 @@ void create_ncc_html (misc_t *misc)
       free (str);
       } // write label
 
-      xmlTextWriterWriteString (writer, BAD_CAST "\n         ");
       xmlTextWriterEndElement (writer);
       xmlTextWriterEndElement (writer);
    } // for
@@ -414,7 +414,8 @@ daisy_t *create_daisy_struct (misc_t *misc,
          fflush (stdout);
       } // if
       misc->total_items = handle_ncc_html (misc, my_attribute, daisy);
-      return calloc (misc->total_items + 1, sizeof (daisy_t));
+      return (daisy_t *) calloc ((size_t) misc->total_items + 1,
+                                 sizeof (daisy_t));
    } // if ncc.html
 
 // handle *.ncx
@@ -433,7 +434,8 @@ daisy_t *create_daisy_struct (misc_t *misc,
    {
       create_ncc_html (misc);
       misc->total_items = handle_ncc_html (misc, my_attribute, daisy);
-      return (daisy_t *) calloc (misc->total_items + 1, sizeof (daisy_t));
+      return (daisy_t *) calloc ((size_t) misc->total_items + 1,
+                                 sizeof (daisy_t));
    } // if
 
 // count items in opf
@@ -443,34 +445,39 @@ daisy_t *create_daisy_struct (misc_t *misc,
       fflush (stdout);
    } // if
    misc->items_in_opf = misc->no_smil_items = 0;
+
    doc = htmlParseFile (misc->opf_name, "UTF-8");
    ptr = xmlReaderWalker (doc);
+   int s = 0;
    while (1)
    {
       if (! get_tag_or_label (misc, my_attribute, ptr))
          break;
       if (strcasestr (my_attribute->media_type, "smil"))
-         misc->items_in_opf++;
+         s++;
+   } // while
+   xmlTextReaderClose (ptr);
+
+   ptr = xmlReaderWalker (doc);
+   int x = 0;
+   while (1)
+   {
+      if (! get_tag_or_label (misc, my_attribute, ptr))
+         break;
+      if (strcasestr (my_attribute->media_type, "xml"))
+      {
+         if (strcasestr (my_attribute->media_type, "smil"))
+            continue;
+         else
+            x++;;
+      } // if
    } // while
    xmlTextReaderClose (ptr);
    xmlFreeDoc (doc);
-
-   if (misc->items_in_opf == 0)
-   {
-      doc = htmlParseFile (misc->opf_name, "UTF-8");
-      ptr = xmlReaderWalker (doc);
-
-      misc->no_smil_items = 1;
-      while (1)              
-      {
-         if (! get_tag_or_label (misc, my_attribute, ptr))
-            break;
-         if (strcasestr (my_attribute->media_type, "xhtml+xml"))
-            misc->items_in_opf++;
-     } // while
-      xmlTextReaderClose (ptr);
-      xmlFreeDoc (doc);
-   } // if
+   if (s >= x)
+      misc->items_in_opf = s;
+   else
+      misc->items_in_opf = x;
 
    if (misc->verbose)
       printf ("%d\n", misc->items_in_opf);
@@ -509,12 +516,12 @@ daisy_t *create_daisy_struct (misc_t *misc,
    misc->total_items = misc->items_in_ncx;
    if (misc->items_in_opf > misc->items_in_ncx)
       misc->total_items = misc->items_in_opf;
-   switch (chdir (misc->daisy_mp));
+   chdir (misc->daisy_mp);
    snprintf (misc->eBook_speaker_txt, MAX_STR,
              "%s/eBook-speaker.txt", misc->daisy_mp);
    snprintf (misc->tmp_wav, MAX_STR,
              "%s/eBook-speaker.wav", misc->daisy_mp);
-   return (daisy_t *) calloc (misc->total_items + 1, sizeof (daisy_t));
+   return (daisy_t *) calloc ((size_t) misc->total_items + 1, sizeof (daisy_t));
 } // create_daisy_struct
 
 void make_tmp_dir (misc_t *misc)
@@ -621,8 +628,8 @@ void get_attributes (misc_t *misc, my_attribute_t *my_attribute,
       char name[MAX_STR], content[MAX_STR];
 
       *name = 0;
-      strncpy (attr, (char *) xmlTextReaderGetAttribute
-               (ptr, (const xmlChar *) "name"), MAX_STR - 1);
+      strcpy (attr, (char *) xmlTextReaderGetAttribute
+               (ptr, (const xmlChar *) "name"));
       if (strcmp (attr, "(null)"))
          strncpy (name, attr, MAX_STR - 1);
       *content = 0;
@@ -631,11 +638,11 @@ void get_attributes (misc_t *misc, my_attribute_t *my_attribute,
       if (strcmp (attr, "(null)"))
          strncpy (content, attr, MAX_STR - 1);
       if (strcasestr (name, "dc:format"))
-         strncpy (misc->daisy_version, content, MAX_STR - 1);
+         strcpy (misc->daisy_version, content);
       if (strcasestr (name, "dc:title") && ! *misc->daisy_title)
       {
-         strncpy (misc->daisy_title, content, MAX_STR - 1);
-         strncpy (misc->bookmark_title, content, MAX_STR - 1);
+         strcpy (misc->daisy_title, content);
+         strcpy (misc->bookmark_title, content);
          if (strchr (misc->bookmark_title, '/'))
             *strchr (misc->bookmark_title, '/') = '-';
       } // if
@@ -658,7 +665,7 @@ void get_attributes (misc_t *misc, my_attribute_t *my_attribute,
       if (strcasestr (name, "dtb:totalTime")  ||
           strcasestr (name, "ncc:totalTime"))
       {
-         strncpy (misc->ncc_totalTime, content, MAX_STR - 1);
+         strcpy (misc->ncc_totalTime, content);
          if (strchr (misc->ncc_totalTime, '.'))
             *strchr (misc->ncc_totalTime, '.') = 0;
       } // if
@@ -677,13 +684,9 @@ void get_attributes (misc_t *misc, my_attribute_t *my_attribute,
       strncpy (my_attribute->smilref, attr, MAX_STR - 1);
    snprintf (attr, MAX_STR - 1, "%s", (char *)
            xmlTextReaderGetAttribute (ptr,
-                                      (const xmlChar *) "pulseaudio_device"));
+                        (const xmlChar *) "current_sink"));
    if (strcmp (attr, "(null)"))
-      misc->pulseaudio_device = atoi (attr);
-   snprintf (attr, MAX_STR - 1, "%s", (char *)
-        xmlTextReaderGetAttribute (ptr, (const xmlChar *) "ocr_language"));
-   if (strcmp (attr, "(null)"))
-      strncpy (misc->ocr_language, attr, 5);
+      misc->current_sink = atoi (attr);
    snprintf (attr, MAX_STR - 1, "%s", (char *)
              xmlTextReaderGetAttribute (ptr, (const xmlChar *) "cd_dev"));
    if (strcmp (attr, "(null)"))
@@ -692,7 +695,7 @@ void get_attributes (misc_t *misc, my_attribute_t *my_attribute,
              xmlTextReaderGetAttribute (ptr, (const xmlChar *) "speed"));
    if (strcmp (attr, "(null)"))
    {
-      misc->speed = atof (attr);
+      misc->speed = (float) atof (attr);
       if (misc->speed <= 0.1 || misc->speed > 2)
          misc->speed = 1.0;
    } // if
@@ -705,7 +708,7 @@ void get_attributes (misc_t *misc, my_attribute_t *my_attribute,
    snprintf (attr, MAX_STR - 1, "%s", (char *)
              xmlTextReaderGetAttribute (ptr, (const xmlChar *) "tts"));
    if (strcmp (attr, "(null)"))
-      misc->attr_tts_no = atof ((char *) attr);
+      misc->attr_tts_no = atoi ((char *) attr);
    snprintf (attr, MAX_STR - 1, "%s", (char *)
              xmlTextReaderGetAttribute (ptr, (const xmlChar *) "toc"));
    if (strcmp (attr, "(null)"))
@@ -780,13 +783,13 @@ int get_tag_or_label (misc_t *misc, my_attribute_t *my_attribute,
       return 1;
    case XML_READER_TYPE_END_ELEMENT:
       snprintf (misc->tag, MAX_TAG - 1, "/%s",
-                (char *) xmlTextReaderName (reader));
+                xmlTextReaderName (reader));
       return 1;
    case XML_READER_TYPE_TEXT:
    case XML_READER_TYPE_CDATA:
    case XML_READER_TYPE_SIGNIFICANT_WHITESPACE:
    {
-      int x;
+      long x;
       size_t phrase_len;
 
       phrase_len = strlen ((char *) xmlTextReaderConstValue (reader));
@@ -799,9 +802,9 @@ int get_tag_or_label (misc_t *misc, my_attribute_t *my_attribute,
             break;
       } // while
       misc->label = realloc (misc->label, phrase_len + 1);
-      snprintf (misc->label, phrase_len - x + 1, "%s", (char *)
+      snprintf (misc->label, phrase_len - (size_t) x + 1, "%s", (char *)
                xmlTextReaderValue (reader) + x);
-      for (x = strlen (misc->label) - 1; x >= 0 && isspace (misc->label[x]);
+      for (x = (long) strlen (misc->label) - 1; x >= 0 && isspace (misc->label[x]);
            x--)
          misc->label[x] = 0;
       for (x = 0; misc->label[x] > 0; x++)
@@ -820,7 +823,7 @@ int get_tag_or_label (misc_t *misc, my_attribute_t *my_attribute,
 } // get_tag_or_label
 
 void go_to_page_number (misc_t *misc, my_attribute_t *my_attribute,
-                          daisy_t *daisy)
+                        daisy_t *daisy, audio_info_t *sound_devices)
 {
    char pn[15];
 
@@ -840,8 +843,8 @@ void go_to_page_number (misc_t *misc, my_attribute_t *my_attribute,
    if (atoi (pn) == 0 || atoi (pn) > misc->total_pages)
    {
       beep ();
-      pause_resume (misc, my_attribute, daisy);
-      pause_resume (misc, my_attribute, daisy);
+      pause_resume (misc, my_attribute, daisy, sound_devices);
+      pause_resume (misc, my_attribute, daisy, sound_devices);
       return;
    } // if
 
@@ -897,84 +900,23 @@ void go_to_page_number (misc_t *misc, my_attribute_t *my_attribute,
                          daisy[misc->playing].xml_anchor);
          return;
       } // if current_page_number >  pn
-      get_next_phrase (misc, my_attribute, daisy, 0);
+      get_next_phrase (misc, my_attribute, daisy, sound_devices, 0); 
    } // while
 } // go_to_page_number
 
-void check_pulseaudio_device (misc_t *misc, daisy_t *daisy)
+void get_list_of_sound_devices (misc_t *misc, audio_info_t *sound_devices)
 {
-   int n, found;
+   int g, sink, found;
+   size_t bytes;
+   char *str, *orig_language;
    struct group *grp;
-   char *str;
-   size_t len = 0;
    FILE *p;
 
    grp = getgrnam ("audio");
    found = 0;
-   for (n = 0; grp->gr_mem[n]; n++)
+   for (g = 0; grp->gr_mem[g]; g++)
    {
-      if (strcmp (grp->gr_mem[n], cuserid (NULL)) == 0)
-      {
-         found = 1;
-         break;
-      } // if
-   } // for
-   if (found == 0)
-   {
-      beep ();
-      endwin ();
-      printf ("You need to be a member of the group \"audio\".\n");
-      printf ("Please give the following command:\n");
-      printf ("\n   sudo gpasswd -a <login-name> audio\n\n");
-      printf ("Logout and login again and you can use daisy-player.\n");
-      _exit (EXIT_FAILURE);
-   } // if
-
-   if (misc->pulseaudio_device < 0)
-   {
-      select_next_output_device (misc, daisy);
-      return;
-   } // if
-   if ((p = popen ("LANG=C /usr/bin/pactl list sinks", "r")) == NULL)
-   {
-      beep ();
-      endwin ();
-      printf ("%s\n",
-      "Be sure the package pulseaudio-utils is installed onto your system.");
-      _exit (EXIT_FAILURE);
-   } // if
-
-   n = -1;
-   str = NULL;
-   len = 0;
-   while (! misc->term_signaled)
-   {
-      if (getline (&str, &len, p) == -1)
-         break;
-      *strchr (str, '\n') = 0; // remove new_line
-      if (strncasecmp (str, "Sink #", 6) != 0)
-         continue;
-      str += 6;
-      if (atoi (str) == misc->pulseaudio_device)
-      {
-         pclose (p);
-         return;
-      } // if
-   } // while
-   pclose (p);
-   select_next_output_device (misc, daisy);
-} // check_pulseaudio_device
-
-void select_next_output_device (misc_t *misc, daisy_t *daisy)
-{
-   int current_sink, n, found;
-   struct group *grp;
-
-   grp = getgrnam ("audio");
-   found = 0;
-   for (n = 0; grp->gr_mem[n]; n++)
-   {
-      if (strcmp (grp->gr_mem[n], cuserid (NULL)) == 0)
+      if (strcmp (grp->gr_mem[g], cuserid (NULL)) == 0)
       {
          found = 1;
          break;
@@ -992,164 +934,272 @@ void select_next_output_device (misc_t *misc, daisy_t *daisy)
       _exit (EXIT_FAILURE);
    } // if
 
-   wclear (misc->screenwin);
-   wprintw (misc->screenwin, "\n%s\n\n", gettext ("Select a soundcard:"));
+// first sink is the ALSA default device
+   strcpy (sound_devices[0].device, "default");
+   strcpy (sound_devices[0].type, "alsa");
+   strcpy (sound_devices[0].name, "(alsa) default sound device");
 
-   nodelay (misc->screenwin, FALSE);
-   current_sink = -1;
-   while (! misc->term_signaled)
+// force english output
+   orig_language = strdup (setlocale (LC_ALL, ""));
+   setlocale (LC_ALL, "en_GB.UTF-8");
+   if (! (p = popen
+       ("/usr/bin/amixer -D default get Master playback", "r")))
    {
+      beep ();
+      endwin ();
+      printf ("%s\n",
+   "Be sure the package alsa-utils is installed onto your system.");
+      remove_tmp_dir (misc);
+      setlocale (LC_ALL, orig_language);
+      _exit (EXIT_FAILURE);
+   } // if
+   setlocale (LC_ALL, orig_language);
+   while (1)
+   {
+      str = NULL;
+      bytes = 0;
+      if (getline (&str, &bytes, p) == -1)
+         break;
+      if (strcasestr (str, "playback") == NULL)
+         continue;
+      if (strchr (str, '[') == NULL)
+         continue;
+      str = strchr (str, '[') + 1;
+      strncpy (sound_devices[0].volume, str, 4);
+      if (strchr (sound_devices[0].volume, ']'))
+         *strchr (sound_devices[0].volume, ']') = 0;
+      strcpy (sound_devices[0].muted, strrchr (str, '[') + 1);
+      *strchr (sound_devices[0].muted, ']') = 0;
+      if (strcmp (sound_devices[0].muted, "on") == 0)
+         strcpy (sound_devices[0].muted, "no");
+      if (strcmp (sound_devices[0].muted, "off") == 0)
+         strcpy (sound_devices[0].muted, "yes");
+   } // while
+   pclose (p);
+
+// now find all other ALSA playback devices
+   misc->total_sinks = alsa_ctl (misc, ALSA_LIST, 1, sound_devices);
+   sink = misc->total_sinks;
+
+// and as last find all pulseaudio devices
+// force english output
+   orig_language = strdup (setlocale (LC_ALL, ""));
+   setlocale (LC_ALL, "en_GB.UTF-8");
+   if ((p = popen ("/usr/bin/pactl list sinks", "r")) == NULL)
+   {
+      beep ();
+      endwin ();
+      printf ("%s\n",
+   "Be sure the package pulseaudio-utils is installed onto your system.");
+      remove_tmp_dir (misc);
+      setlocale (LC_ALL, orig_language);
+      _exit (EXIT_FAILURE);
+   } // if
+   setlocale (LC_ALL, orig_language);
+   while (1)
+   {
+      size_t len;
       char *str;
-      size_t len = 0;
-      int total;
-      FILE *p;
 
-      if ((p = popen ("LANGUAGE=C /usr/bin/pactl list sinks", "r")) == NULL)
+      str = NULL;
+      len = 0;
+      if (getline (&str, &len, p) == -1)
+         break;
+      if (strncasecmp (str, "Sink #", 6) == 0)
       {
-         beep ();
-         endwin ();
-         printf ("%s\n",
-      "Be sure the package pulseaudio-utils is installed onto your system.");
-         remove_tmp_dir (misc);
-         _exit (EXIT_FAILURE);
-      } // if
+         int x;
 
-      n = -1;
-      while (! misc->term_signaled)
+         x = 0;
+         strcpy (sound_devices[sink].device, str + 6);
+         while (1)
+         {
+            if (isspace (sound_devices[sink].device[x]) != 0)
+            {
+               sound_devices[sink].device[x] = 0;
+               break;
+            } // if
+            x++;
+         } // while
+         strcpy (sound_devices[sink].type, "pulseaudio");
+      } // if device
+
+      if (strcasestr (str, "alsa.card_name"))
       {
-         str = NULL;
-         len = 0;
-         if (getline (&str, &len, p) == -1)
+         long x;
+
+         str = strchr (str, '=') + 1;
+         while (1)
+         {
+            if (isspace (*str) == 0)
+               break;
+            str++;
+         } // while
+         for (x = (long) strlen (str) - 1; x > 0; x--)
+         {
+            if (isspace (str[x]) == 0)
+               break;
+         } // for
+         str[x + 1] = 0;
+         sprintf (sound_devices[sink].name, "(pulseaudio) %s", str);
+      } // if name
+
+      if (strcasestr (str, "Mute:"))
+      {
+         str = strchr (str, ':') + 1;
+         while (1)
+         {
+            if (isspace (*str) == 0)
+               break;
+            str++;
+         } // while
+         while (1)
+         {
+            long x;
+
+            x = (long) strlen (str) - 1;
+            if (isspace (str[x]) == 0)
+            {
+               str[x + 1] = 0;
+               break;
+            } // if
+            str[x] = 0;
+         } // while
+         strcpy (sound_devices[sink].muted, str);
+      } // if mute
+
+      if (strcasestr (str, "Volume:"))
+      {
+         long x;
+
+         if (strcasestr (str, "Base"))
+            continue;
+         for (x = (long) strlen (str); x >= 0; x--)
+            if (str[x] == '%')
+               break;
+         str[x + 1] = 0;
+         strcpy (sound_devices[sink].volume, strrchr (str, ' ') + 1);
+      } // if volume
+   } // while
+   pclose (p);
+   misc->total_sinks = ++sink;
+} // get_list_of_sound_devices
+
+void select_next_output_device (misc_t *misc, daisy_t *daisy,
+           audio_info_t *sound_devices)
+{
+   int n;
+
+   while (! misc->term_signaled) // forever if not signaled
+   {
+      get_list_of_sound_devices (misc, sound_devices);
+      wclear (misc->screenwin);
+      wprintw (misc->screenwin, "\n%s\n\n", gettext ("Select a soundcard:"));
+      for (n = 0; n < misc->total_sinks; n++)
+      {
+         mvwprintw (misc->screenwin, n + 3, 3, "%s", sound_devices[n].name);
+         mvwprintw (misc->screenwin, n + 3, 57, " Mute: %3s",
+                    sound_devices[n].muted);
+         wprintw (misc->screenwin, " Volume: %s\n",
+                  sound_devices[n].volume);
+      } // for
+
+// place the cursor on the current device/type
+      for (int c_s = 0; c_s < misc->total_sinks; c_s++)
+      {
+         if (c_s == misc->current_sink)
+         {
+            misc->current_sink = c_s;
             break;
-         if (strncasecmp (str, "Sink #", 6) == 0)
-         {
-            memset (sink_info[++n], ' ', 70);
-            len = strlen (str) - 6 - 1;
-// allow only up to 3 digits
-            if (len > 3)
-               len = 3;
-            strncpy (sink_info[n], &str[6], len);
-            if (current_sink < 0 &&
-                atoi (sink_info[n]) == misc->pulseaudio_device)
-               current_sink = n;
          } // if
-
-/* disabled.
-   Requested by Didier Spaier <didier@slint.fr>
-         if (strstr (str, "Description:"))
-         {
-            int x;
-
-            strcpy (sink_info[n] + 4, str + 14);
-            for (x = strlen (sink_info[n]) - 1; x < 70; x++)
-               sink_info[n][x] = ' ';
-            sink_info[n][x] = 0 ;
-         } // if
-*/
-         if (strcasestr (str, "alsa.card_name"))
-         {
-             if (strlen (str) > 19)
-               str += 19;
-            len = strlen (str) + 3;
-            memcpy (sink_info[n] + 4, str, len);
-            memset (sink_info[n] + len, ' ', 53 - len);
-         } // if
-         if (strcasestr (str, "Mute:"))
-         {
-            strcpy (sink_info[n] + 53, strcasestr (str, "Mute:"));
-            sink_info[n][52] = ' ';
-         } // if
-         if (strcasestr (str, "Volume:"))
-         {
-            int x;
-
-            if (strcasestr (str, "Base"))
-               continue;
-            sink_info[n][strlen (sink_info[n]) - 1] = ' ';
-            sink_info[n][strlen (sink_info[n])] = ' ';
-            strcpy (sink_info[n] + 63, strstr (str, "Volume:"));
-            sink_info[n][70] = 0;
-            for (x = strlen (str); x >= 0; x--)
-               if (str[x] == '%')
-                  break;
-            str[x + 1] = 0;
-            for (x = strlen (str); x >= 0; x--)
-               if (str[x] == ' ')
-                  break;
-            strcat (sink_info[n], str + x);
-         } // if
-      } // while
-      if (current_sink == -1)
-         current_sink = 0;
-      total = n + 1;
-      free (str);
-      pclose (p);
-
-/* The build-in pactl function with argument list gives problems.
-   So for the time being the external application pactl is used instead.
-
-      memcpy (sink_info, pactl ("list", "0", "0"), 1000);
-*/
-      for (n = 0; n < total; n++)
-         mvwprintw (misc->screenwin, n + 3, 3, "%s ", sink_info[n]);
-      wmove (misc->screenwin, current_sink + 3, 2);
+      } // for
+      if (misc->current_sink == misc->total_sinks)
+         misc->current_sink = 0;
+      wmove (misc->screenwin, misc->current_sink + 3, 2);
       wrefresh (misc->screenwin);
 
       switch (wgetch (misc->screenwin))
       {
       case 13:
-         misc->pulseaudio_device = atoi (sink_info[current_sink]);
          view_screen (misc, daisy);
-         nodelay (misc->screenwin, TRUE);
          return;
       case KEY_DOWN:
-         if (++current_sink >= total)
-            current_sink = 0;
+         if (++misc->current_sink >= misc->total_sinks)
+            misc->current_sink = 0;
+         wmove (misc->screenwin, misc->current_sink + 3, 2);
+         wrefresh (misc->screenwin);
          break;
       case KEY_UP:
-         if (--current_sink < 0)
-            current_sink = total - 1;
+         if (--misc->current_sink < 0)
+            misc->current_sink = misc->total_sinks - 1;
+         wmove (misc->screenwin, misc->current_sink + 3, 2);
+         wrefresh (misc->screenwin);
          break;
       case 'm':
-         if (fork () == 0)
+         if (strcmp (sound_devices[misc->current_sink].type, "alsa") == 0)
          {
-            char dev[5];
-
-            reset_term_signal_handlers_after_fork ();
-            sprintf (dev, "%d", atoi (sink_info[current_sink]));
-            (void) pactl ("set-sink-mute", dev, "toggle");
-            _exit (EXIT_SUCCESS);
+            sprintf (misc->cmd, "/usr/bin/amixer -q -D %s set Master playback toggle",
+                     sound_devices[misc->current_sink].device);
+            system (misc->cmd);
+         }
+         else
+         {
+            if (fork () == 0)
+            {
+               reset_term_signal_handlers_after_fork ();
+               pactl ("set-sink-mute",
+                      sound_devices[misc->current_sink].device,
+                      "toggle");
+               _exit (EXIT_SUCCESS);
+            } // if
          } // if
          break;
       case 'q':
          view_screen (misc, daisy);
-         if (misc->pulseaudio_device < 0)
-            misc->pulseaudio_device = 0;
+         if (misc->current_sink == -1)
+            misc->current_sink = 0;
          return;
       case 'v':
       case '1':
-         if (fork () == 0)
+         if (strcmp (sound_devices[misc->current_sink].type, "alsa") == 0)
          {
-            char dev[5];
-
-            reset_term_signal_handlers_after_fork ();
-            sprintf (dev, "%d", atoi (sink_info[current_sink]));
-            (void) pactl ("set-sink-volume", dev, "-5%");
-            _exit (EXIT_SUCCESS);
+            sprintf (misc->cmd, "/usr/bin/amixer -q -D %s set Master playback 5%%-",
+                     sound_devices[misc->current_sink].device);
+            system (misc->cmd);
+         }
+         else
+         {
+            if (fork () == 0)
+            {
+               reset_term_signal_handlers_after_fork ();
+               pactl ("set-sink-volume",
+                      sound_devices[misc->current_sink].device,
+                      "-5%");
+               _exit (EXIT_SUCCESS);
+            } // if
          } // if
          break;
       case 'V':
       case '7':
-         if (fork () == 0)
+         if (strcmp (sound_devices[misc->current_sink].type, "alsa") == 0)
          {
-            char dev[5];
-
-            reset_term_signal_handlers_after_fork ();
-            sprintf (dev, "%d", atoi (sink_info[current_sink]));
-            (void) pactl ("set-sink-volume", dev, "+5%");
-            _exit (EXIT_SUCCESS);
+            sprintf (misc->cmd, "/usr/bin/amixer -q -D %s set Master playback 5%%+",
+                     sound_devices[misc->current_sink].device);
+            system (misc->cmd);
+         }
+         else
+         {
+            if (fork () == 0)
+            {
+               reset_term_signal_handlers_after_fork ();
+               pactl ("set-sink-volume",
+                      sound_devices[misc->current_sink].device,
+                      "+5%");
+               _exit (EXIT_SUCCESS);
+            } // if
          } // if
          break;
+      case ERR:
+         break;
       default:
          beep ();
          break;
@@ -1166,7 +1216,7 @@ void select_next_output_device (misc_t *misc, daisy_t *daisy)
       select (1, &rfds, NULL, NULL, &tv);
    } // while
 } // select_next_output_device
-          
+
 // Useful to reset handlers in fork()ed children
 void reset_term_signal_handlers_after_fork (void)
 {
@@ -1179,7 +1229,6 @@ void free_all (misc_t *misc, my_attribute_t *my_attribute, daisy_t *daisy)
 {
    free (misc->src_dir);
    free (misc->daisy_mp);
-   free (misc->tmp_dir);
    free (misc->label);
    free (my_attribute->id);
    free (my_attribute->idref);
